/***************************************************************************
 *   Copyright (C) 2004 by Claudio Attaccalite                             *
 *   claudio@freescience.info                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "MyBlas.h"

/** Double Precision BLAS **/

void dgemm(double alpha,DMatrix &A,DMatrix &B,double beta,DMatrix &C)
{
int m=A.dim1();
int n=B.dim2();
int k=B.dim1();
int ldc=C.dim1();
F77_FUNC(dgemm,DGEMM)("N","N",&m,&n,&k,&alpha,A.array(),&m,B.array(),&k,&beta,C.array(),&ldc);
}

void dgemm_n_t(double alpha,DMatrix &A,DMatrix &B,double beta,DMatrix &C)
{
int m=A.dim1();
int n=B.dim1();
int k=A.dim2();
int ldc=C.dim1();
F77_FUNC(dgemm,DGEMM)("N","T",&m,&n,&k,&alpha,A.array(),&m,B.array(),&n,&beta,C.array(),&ldc);
}

void dgemm_t_t(double alpha,DMatrix &A,DMatrix &B,double beta,DMatrix &C)
{
int m=A.dim2();
int n=B.dim1();
int k=A.dim1();
int ldc=C.dim1();
F77_FUNC(dgemm,DGEMM)("T","T",&m,&n,&k,&alpha,A.array(),&k,B.array(),&n,&beta,C.array(),&ldc);
}

void dgemm_t_n(double alpha,DMatrix &A,DMatrix &B,double beta,DMatrix &C)
{
int m=A.dim2();
int k=A.dim1();
int n=B.dim2();
int ldb=B.dim1();
int ldc=C.dim1();
F77_FUNC(dgemm,DGEMM)("T","N",&m,&n,&k,&alpha,A.array(),&k,B.array(),&ldb,&beta,C.array(),&ldc);
}


void dgemv(double alpha,DMatrix &A,DVector &X,double beta,DVector &Y)
{
int m=A.dim2();
int n=A.dim1();
int one=1;
F77_FUNC(dgemv,DGEMV)("N",&n,&m,&alpha,A.array(),&n,X.array(),&one,&beta,Y.array(),&one);
}

void dgemv_t(double alpha,DMatrix &A,DVector &X,double beta,DVector &Y)
{
int n=A.dim2();
int m=A.dim1();
int one=1;
F77_FUNC(dgemv,DGEMV)("T",&m,&n,&alpha,A.array(),&m,X.array(),&one,&beta,Y.array(),&one);
}

/** Single Precision BLAS **/

void sgemm(float alpha,SMatrix &A,SMatrix &B,float beta,SMatrix &C)
{
int m=A.dim1();
int n=B.dim2();
int k=B.dim1();
int ldc=C.dim1();
F77_FUNC(sgemm,SGEMM)("N","N",&m,&n,&k,&alpha,A.array(),&m,B.array(),&k,&beta,C.array(),&ldc);
}

void sgemm_n_t(float alpha,SMatrix &A,SMatrix &B,float beta,SMatrix &C)
{
int m=A.dim1();
int n=B.dim1();
int k=A.dim2();
int ldc=C.dim1();
F77_FUNC(sgemm,SGEMM)("N","T",&m,&n,&k,&alpha,A.array(),&m,B.array(),&n,&beta,C.array(),&ldc);
}

void sgemm_t_t(float alpha,SMatrix &A,SMatrix &B,float beta,SMatrix &C)
{
int m=A.dim2();
int n=B.dim1();
int k=A.dim1();
int ldc=C.dim1();
F77_FUNC(sgemm,SGEMM)("T","T",&m,&n,&k,&alpha,A.array(),&k,B.array(),&n,&beta,C.array(),&ldc);
}

void sgemm_t_n(float alpha,SMatrix &A,SMatrix &B,float beta,SMatrix &C)
{
int m=A.dim2();
int k=A.dim1();
int n=B.dim2();
int ldb=B.dim1();
int ldc=C.dim1();
F77_FUNC(sgemm,SGEMM)("T","N",&m,&n,&k,&alpha,A.array(),&k,B.array(),&ldb,&beta,C.array(),&ldc);
}

void sgemv(float alpha,SMatrix &A,SVector &X,float beta,SVector &Y)
{
int n=A.dim2();
int m=A.dim1();
int one=1;
F77_FUNC(sgemv,SGEMV)("N",&n,&m,&alpha,A.array(),&n,X.array(),&one,&beta,Y.array(),&one);
}

void sgemv_t(float alpha,SMatrix &A,SVector &X,float beta,SVector &Y)
{
int n=A.dim2();
int m=A.dim1();
int one=1;
F77_FUNC(sgemv,SGEMV)("T",&n,&m,&alpha,A.array(),&n,X.array(),&one,&beta,Y.array(),&one);
}


/******** Complex BLAS ***************/
void zgemv(complex<double> alpha,ZMatrix &A,ZVector &X,complex<double> beta,ZVector &Y)
{
int m=A.dim2();
int n=A.dim1();
int one=1;
F77_FUNC(zgemv,ZGEMV)("N",&n,&m,(double*)&alpha,(double*)A.array(),&n,(double*)X.array(),&one,(double*)&beta,(double*)Y.array(),&one);
}

void zgemv_t(complex<double> alpha,ZMatrix &A,ZVector &X,complex<double> beta,ZVector &Y)
{
int n=A.dim2();
int m=A.dim1();
int one=1;
F77_FUNC(dgemv,DGEMV)("T",&m,&n,(double*)&alpha,(double*)A.array(),&m,(double*)X.array(),&one,(double*)&beta,(double*)Y.array(),&one);
}

void zgemv_c(complex<double> alpha,ZMatrix &A,ZVector &X,complex<double> beta,ZVector &Y)
{
int n=A.dim2();
int m=A.dim1();
int one=1;
F77_FUNC(dgemv,DGEMV)("C",&m,&n,(double*)&alpha,(double*)A.array(),&m,(double*)X.array(),&one,(double*)&beta,(double*)Y.array(),&one);
}

void zgemm(complex<double> alpha,ZMatrix &A,ZMatrix &B,complex<double> beta,ZMatrix &C)
{
int m=A.dim1();
int n=B.dim2();
int k=B.dim1();
int ldc=C.dim1();
F77_FUNC(zgemm,ZGEMM)("N","N",&m,&n,&k,(double*)&alpha,(double*)A.array(),&m,(double*)B.array(),&k,(double*)&beta,(double*)C.array(),&ldc);
}

void zgemm_n_t(complex<double> alpha,ZMatrix &A,ZMatrix &B,complex<double> beta,ZMatrix &C)
{
int m=A.dim1();
int n=B.dim1();
int k=A.dim2();
int ldc=C.dim1();
F77_FUNC(zgemm,ZGEMM)("N","T",&m,&n,&k,(double*)&alpha,(double*)A.array(),&m,(double*)B.array(),&n,(double*)&beta,(double*)C.array(),&ldc);
}

void zgemm_t_t(complex<double> alpha,ZMatrix &A,ZMatrix &B,complex<double> beta,ZMatrix &C)
{
int m=A.dim2();
int n=B.dim1();
int k=A.dim1();
int ldc=C.dim1();
F77_FUNC(zgemm,ZGEMM)("T","T",&m,&n,&k,(double*)&alpha,(double*)A.array(),&k,(double*)B.array(),&n,(double*)&beta,(double*)C.array(),&ldc);
}

void zgemm_t_n(complex<double> alpha,ZMatrix &A,ZMatrix &B,complex<double> beta,ZMatrix &C)
{
int m=A.dim2();
int k=A.dim1();
int n=B.dim2();
int ldb=B.dim1();
int ldc=C.dim1();
F77_FUNC(zgemm,ZGEMM)("T","N",&m,&n,&k,(double*)&alpha,(double*)A.array(),&k,(double*)B.array(),&ldb,(double*)&beta,(double*)C.array(),&ldc);
}
/******* More with C *********/

void zgemm_n_c(complex<double> alpha,ZMatrix &A,ZMatrix &B,complex<double> beta,ZMatrix &C)
{
int m=A.dim1();
int n=B.dim1();
int k=A.dim2();
int ldc=C.dim1();
F77_FUNC(zgemm,ZGEMM)("N","C",&m,&n,&k,(double*)&alpha,(double*)A.array(),&m,(double*)B.array(),&n,(double*)&beta,(double*)C.array(),&ldc);
}

void zgemm_c_c(complex<double> alpha,ZMatrix &A,ZMatrix &B,complex<double> beta,ZMatrix &C)
{
int m=A.dim2();
int n=B.dim1();
int k=A.dim1();
int ldc=C.dim1();
F77_FUNC(zgemm,ZGEMM)("C","C",&m,&n,&k,(double*)&alpha,(double*)A.array(),&k,(double*)B.array(),&n,(double*)&beta,(double*)C.array(),&ldc);
}

void zgemm_c_n(complex<double> alpha,ZMatrix &A,ZMatrix &B,complex<double> beta,ZMatrix &C)
{
int m=A.dim2();
int k=A.dim1();
int n=B.dim2();
int ldb=B.dim1();
int ldc=C.dim1();
F77_FUNC(zgemm,ZGEMM)("C","N",&m,&n,&k,(double*)&alpha,(double*)A.array(),&k,(double*)B.array(),&ldb,(double*)&beta,(double*)C.array(),&ldc);
}

void zgemm_c_t(complex<double> alpha,ZMatrix &A,ZMatrix &B,complex<double> beta,ZMatrix &C)
{
int m=A.dim2();
int n=B.dim1();
int k=A.dim1();
int ldc=C.dim1();
F77_FUNC(zgemm,ZGEMM)("C","T",&m,&n,&k,(double*)&alpha,(double*)A.array(),&k,(double*)B.array(),&n,(double*)&beta,(double*)C.array(),&ldc);
}

void zgemm_t_c(complex<double> alpha,ZMatrix &A,ZMatrix &B,complex<double> beta,ZMatrix &C)
{
int m=A.dim2();
int n=B.dim1();
int k=A.dim1();
int ldc=C.dim1();
F77_FUNC(zgemm,ZGEMM)("T","C",&m,&n,&k,(double*)&alpha,(double*)A.array(),&k,(double*)B.array(),&n,(double*)&beta,(double*)C.array(),&ldc);
}

