/***************************************************************************
 *   Copyright (C) 2004 by Claudio Attaccalite                             *
 *   claudio@freescience.info                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef MyLapack 
#define MyLapack  "@(#)MyLapack.h $Revision$"
#if HAVE_CONFIG
#include <config.h>
#endif

#include <iostream>
#include "lapack_wrapper.h"
#include "DMatrix.h"
#include "DVector.h"
#include "SMatrix.h"
#include "SVector.h"


/** Double Precision **/

/**
  LU decomposition  for a dense matrix.
  @param A a \f$N \times N\f$ matrix which is destroyed during the operation.
  The resulting LU decompositon is placed here.
  @param ipiv a \f$N\f$ dimensional array which records the row permutation 
  from partial pivoting.
  @return returns false if the calculation is singular and true otherwise
 **/

int DLU_decomp(DMatrix &A,int *ipiv);

/**
  Calculates the inverse and determinant of a matrix using a 
  dense LU solver.  This 
  method scales as \f$O(1 N^3)\f$.
**/

int DLU_invert(DMatrix &A,int *ipiv,double *work,int &lwork);

/**
  Calculates th determinant of a matrix using its LU decomposed matrix 
**/

double DLU_det(DMatrix &A,int *ipiv);

/**
  Solves a system of linear equations using a dense LU solver.
  this method scales as \f$O(\frac{1}{3} N^3)\f$.
**/

int DLU_solve(DMatrix &M,int *ipiv,DVector &b);

/**
  Calculate Eigenvalues and also Eigenvector of a symmetric matrix
**/

int DSYEV(char jobz,char uplo,DMatrix &M,DVector &V,double *work,int &lwork);

/** Single Precision **/

int SLU_decomp(SMatrix &A,int *ipiv);

int SLU_invert(SMatrix &A,int *ipiv,float *work,int &lwork);

float SLU_det(SMatrix &A,int *ipiv);

int SLU_solve(SMatrix &M,int *ipiv,DVector &b);

int SSYEV(char jobz,char uplo,SMatrix &M,SVector &V,float *work,int &lwork);


#endif
